Module: Freno::Throttler::Instrumenter
- Defined in:
- lib/freno/throttler/instrumenter.rb
Overview
An Instrumenter is an object that responds to
instrument(event_name, payload = {}) to receive events from the
throttler.
As an example, in a rails app one could use ActiveSupport::Notifications as an instrumenter and subscribe to the "freno.*" events somewhere else in the application.
Defined Under Namespace
Classes: Noop