Module: Freno::Throttler::CircuitBreaker
- Defined in:
- lib/freno/throttler/circuit_breaker.rb
Overview
A CircuitBreaker is the entry point of the pattern with same name. (see https://martinfowler.com/bliki/CircuitBreaker.html)
Clients that use circuit breakers to add resiliency to their processes
send failure or sucess messages to the CircuitBreaker depending on the
results of the last requests made.
With that information, the circuit breaker determines whether or not to
allow the next request (allow_request?). A circuit is said to be open
when the next request is not allowed; and it's said to be closed when the
next request is allowed.
Defined Under Namespace
Classes: Noop